import numpy as np

def find_bursts(sorted_dict, data_dict, sorting_flag = 'spiketimes_good'):
 
  ##### Find good channels #####
  good_channels = []
  for ele in range(len(data_dict['elenumber'])):
    if len(sorted_dict['spiketimes_good'][ele]) > 1:
      good_channels.append(ele)
      
      
  ##### Calculate time constant for burst detection #####
  spikecount = 0
  for i in good_channels:
    spikecount = spikecount + len(sorted_dict['spiketimes_good'][i])
  if good_channels:
    average_rate_active_electrodes = spikecount / (data_dict['length_recording'] * len(good_channels) )
    th = 0.25 * 1/average_rate_active_electrodes
    print '\n Burst time constant is: ', th
  else:
    th = 0
    
    
  # find whenever spikes are closer together than burst time constant.  
  
  if sorting_flag == 'spiketimes_all':
    spikes = data_dict['spiketimes']
  else:
    spikes = sorted_dict[sorting_flag]

  burst_dict = {}
  for i in range(len(data_dict['elenumber'])):
    burstflag = np.diff(spikes[i][:]) < th
    if len(burstflag >0):
      burstflag_ri = np.append(burstflag[0], burstflag)
      burstflag_le = np.append(burstflag, burstflag[-1])
      burstflag = np.logical_or(burstflag_ri, burstflag_le)
    burstinfo = {'sp_allspikes':spikes[i][:],\
    'sp_inbursts': [spikes[i][s] for s in np.where(burstflag)[0] ],\
    'sp_notinbursts': [spikes[i][s] for s in np.where(~burstflag)[0] ]}

    burst_dict[i] = burstinfo
  burst_dict['th'] = th
  burst_dict['good_channels'] = good_channels
  
  scounter = 0
  for i in range(len(data_dict['elenumber'])):
    scounter = scounter + len(burst_dict[i]['sp_notinbursts'])
  burst_dict['spikes_not_in_bursts'] = scounter
  
  return burst_dict



def extract_bursttimes_from_bursts( burst_dict, data_dict ):
  th = burst_dict['th']

  burst_inf = [[] for j in range(len(data_dict['elenumber']))]

  for i in range(len(data_dict['elenumber'])):
    spikes_on_this_channel_in_bursts = burst_dict[i]['sp_inbursts']
    if len(spikes_on_this_channel_in_bursts)>0:
      s0 = 0
      last_spike_of_burst = 0
      
      for s in range(len(spikes_on_this_channel_in_bursts)-1):
	if (spikes_on_this_channel_in_bursts[s+1] - spikes_on_this_channel_in_bursts[s]) < th:
	  #if there is one flaged spike later and less than th apart, then this spike belongs to burst,
	  last_spike_of_burst = s + 1
	else:
	  #burst ended
	  starttime = spikes_on_this_channel_in_bursts[s0]
	  if last_spike_of_burst > s0:
	    burst_inf[i].append({'time':starttime , 'length':spikes_on_this_channel_in_bursts[last_spike_of_burst] - starttime, 'spikes': 1 + last_spike_of_burst - s0})
	    #print last_spike_of_burst, s0
	  #And a new burst begins with this s
	  last_spike_of_burst = s + 1
	  s0 = s + 1
	  
  return burst_inf



def find_global_bursts( burst_inf, data_dict, acc_time = 1.25 ):
  #next, overlapping bursts are merged:

  #step 1: merge all burst events into one:
  all_bursts = []
  for i in range(len(data_dict['elenumber'])):
    if len(burst_inf[i])>0:
      for s in burst_inf[i]:
	all_bursts.append([s['time'], s['length']])
    
  #step 2: sort them from initial time
  all_bursts.sort(key=lambda x: x[0])


  #step 3: start with first, go length, any other burst in that time*(5/4), add it. See when end is reached.
  global_bursts = []
  if len(all_bursts) > 0:
    t0 = all_bursts[0][0]
    t_end = all_bursts[0][0] + all_bursts[0][1]*acc_time

    for s in range(len(all_bursts)):
      t_new = all_bursts[s][0]
      deltat_new = all_bursts[s][1]

      if t_new < t_end:
	#then the burst extends until here:
	t_end = t_new + deltat_new*acc_time
      else:
	#burst has ended and the next starts
	global_bursts.append({'time':t0 , 'length':t_end - t0})
	
	t0 = all_bursts[s][0]
	t_end = all_bursts[s][0] + all_bursts[s][1]*acc_time
    
  return global_bursts 
      
    
    
    
    
    
    
    
    
    
    
    
    
    
    

