from matplotlib.mlab import PCA
from sklearn.cluster import MeanShift, DBSCAN, estimate_bandwidth
import numpy as np


np.random.seed(0)

def sorting(data_dict, waveformlength = 150, minimum_number = 30) :
  #minimum_number is the minimum number of spikes in a cluster, could also scale with running time minimum_number = 0.2 * fd.metadata_raw['TimeSpan'] or similar
  #waveformlength is the number of points per waveform. typically 150
    waveforms = data_dict['waveforms']
    spiketimes = data_dict['spiketimes']
    N_ele = len(data_dict['elenumber'])
    averaged_waveform_bad = np.zeros((N_ele,waveformlength))
    averaged_waveform_bad_sc = np.zeros((N_ele,waveformlength))
    averaged_waveform_good = np.zeros((N_ele,waveformlength))
    averaged_waveform_good_sc = np.zeros((N_ele,waveformlength))
    good_spikes = [[] for j in range(N_ele)]
    bad_spikes = [[] for j in range(N_ele)]
    principal_components = [[] for j in range(N_ele)]
    event_labels = [[] for j in range(N_ele)]
    gblist = [[] for j in range(N_ele)]
    
    for ele in range(N_ele):
        print ele
        tmp = np.zeros((len(waveforms[ele][:]),waveformlength))
        for j in range(0,len(waveforms[ele][:])):
            tmp[j,:] =  waveforms[ele][j]
        a,b = tmp.shape
        if a>b:
            results = PCA(tmp)
            data = np.array([results.Y[:,0], results.Y[:,1]])
            data = data.transpose()
            principal_components[ele].append(data)

            ########meanShift
            bandwidth = estimate_bandwidth(data, quantile = 0.3)
            ms = MeanShift(cluster_all=True, bandwidth = bandwidth)
            ms.fit(data)
            labels = ms.labels_
            labels_unique = np.unique(labels)
            n_clusters_ = len(labels_unique)
            number_of_points = [ np.sum(np.array(labels == i, dtype = int)) for i in np.sort(labels_unique)]
            labels_sorted = np.argsort(number_of_points)# order from biggest to smallest
            counter = 0
            for i in labels_sorted:
                labels[labels == i] = 100 + counter
                counter = counter + 1
            labels = labels - 100
            for i in np.sort(labels_unique):
                number_of_points = np.sum(np.array(labels == i, dtype = int))
                if number_of_points < minimum_number:
                    labels[labels == i] = -1
            w = [[] for i in labels_unique]
            averaged_waveform = np.zeros((len(labels_unique),waveformlength))
            averaged_waveform_sc = np.zeros((len(labels_unique),waveformlength))
            for i in np.sort(labels_unique):
                number_of_events = np.sum(np.array(labels == i, dtype = int))
                for j in np.arange(0,len(results.Y)):
                    if labels[j] == i:
                        w[i].append(tmp[j,:])
                averaged_waveform[i][:] = np.mean(w[i][:], axis = 0)
            event_labels[ele].append(labels)
            ####### End spikesorting

            # Select good clusters
            w_good = []
            w_bad = []
            for i in np.arange(0,len(results.Y)):
                #~ print ele, np.std(averaged_waveform[labels[i]]), np.max(averaged_waveform[labels[i]])
                if (np.std(averaged_waveform[labels[i]]) < 5*1e-6) and (np.max(averaged_waveform[labels[i]]) < 4*1e-6):
                    bad_spikes[ele].append(spiketimes[ele][i])
                    w_bad.append(tmp[i,:])
                    gblist[ele].append(0)
                else:
                    good_spikes[ele].append(spiketimes[ele][i])
                    w_good.append(tmp[i,:])
                    gblist[ele].append(1)

            averaged_waveform_good[ele][:] = np.mean(w_good, axis = 0)
            averaged_waveform_good_sc[ele][:] = np.std(w_good, axis = 0) 
            averaged_waveform_bad[ele][:] = np.mean(w_bad, axis = 0) 
            averaged_waveform_bad_sc[ele][:] = np.std(w_bad, axis = 0) 


    sorted_dict= {\
    'spiketimes_good': good_spikes,                'spiketimes_bad': bad_spikes,\
    'mean_waveforms_good': averaged_waveform_good, 'std_waveforms_good': averaged_waveform_good_sc,\
    'mean_waveforms_bad': averaged_waveform_bad,   'std_waveforms_bad': averaged_waveform_bad_sc,\
    'principal_components': [gblist, principal_components, labels]}


    return sorted_dict
